import org.opencv.core.*;
import org.opencv.core.Point;
import org.opencv.highgui.HighGui;
import org.opencv.imgcodecs.Imgcodecs;
import org.opencv.imgproc.Imgproc;
import static org.opencv.imgcodecs.Imgcodecs.imwrite;
import java.awt.image.BufferedImage;
import java.awt.image.DataBufferByte;
import java.io.File;
import javax.imageio.ImageIO;

//REFERENCES:
//https://docs.opencv.org/3.4.3/d9/db0/tutorial_hough_lines.
//https://stackoverflow.com/questions/43443309/count-red-pixel-in-a-given-image
//https://www.wikihow.com/Calculate-Percentage-in-Java
//https://riptutorial.com/opencv/example/21963/converting-an-mat-object-to-an-bufferedimage-object



//GOAL for 21st


//Classifier 01
//Have args so can call "java image-identification-classifier01 XX XX"
//args can be parameters in algorthim such as threshold or theta?
//Run on 5000 images.
//Record success rates
//All done with makefile


//But first understand houghline transform
//Know what the algorithm being used is doing.
//MAke constants for this classifier
//Make java be able to run on CMD line

public class Main {

    //GLOBAL_CONSTANTS
    //SHOULD TURN INTO ARGS

    private static  BufferedImage toBufferedImage(Mat mat){
        //MOSTLY COPY PASTE!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        //MOSTLY COPY PASTE!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        //https://riptutorial.com/opencv/example/21963/converting-an-mat-object-to-an-bufferedimage-object
        try{
            int type = BufferedImage.TYPE_3BYTE_BGR;
            int bufferSize = mat.channels() * mat.cols() * mat.rows();
            byte[] b = new byte[bufferSize];
            //get all the pixels
            mat.get(0, 0, b);
            BufferedImage image = new BufferedImage(mat.cols(), mat.rows(), type);
            final byte[] targetPixels = ((DataBufferByte) image.getRaster().getDataBuffer()).getData();
            System.arraycopy(b, 0, targetPixels, 0, b.length);
            return image;
        }
        catch(Exception e){
            System.err.println(e);
        }
        return null;
    }

    private static boolean Classifier(BufferedImage img){
        try {
            //Read file
            //BufferedImage img = ImageIO.read(new File(processedFile));
            int x = img.getWidth();
            int y = img.getHeight();
            int pixelCount  = 0;
            int redCount    = 0;
            float percentage = 0;

            //Go Thru every pixel
            for(int i=0; i < y; i++){
                for(int j=0;j < x; j++){
                    //Get value for current pixels RGB value
                    int currPixelRGB = img.getRGB(j, i);
                    //Check if pixel is red (hex value of red)
                    if(currPixelRGB == 0xFFFF0000){
                        redCount++;
                    }
                    pixelCount++;
                }
            }
            //Calculate percentage of Red in image
            percentage = ((float)redCount/(float)pixelCount)*(float)100;
            //If more than %10 and less than %50 then its sheet music!
            if(percentage > 10 && percentage < 50){ //MAKE THESE CONSTANTS!!
                return true;}
        }
        catch (Exception e) {
            System.err.println(e);
        }
        return false;
    }

    public static void main(String[] args) {
        System.loadLibrary(Core.NATIVE_LIBRARY_NAME);
        try {
            //Variables
            Mat edgesDetected = new Mat();
            Mat edgesDetectedRGB = new Mat();
            Mat forOTSU = new Mat();
            Mat edgesDetectedRGBProb;
            Mat justEdges; //TESTING
            String default_file = "TestImages/houghlineTest.png";
            String filename = ((args.length > 0) ? args[0] : default_file);

            // Load an image
            Mat original = Imgcodecs.imread(filename, Imgcodecs.IMREAD_GRAYSCALE);
            // Edge detection
            //01 CANNY
            //Imgproc.Canny(original, edgesDetected, 50, 200, 3, false);
            //02 BINARYINV
            Imgproc.adaptiveThreshold(original, edgesDetected,255, Imgproc.ADAPTIVE_THRESH_GAUSSIAN_C,Imgproc.THRESH_BINARY_INV,15, 2);
            //03 BINARY
            //Imgproc.adaptiveThreshold(original, edgesDetected,255, Imgproc.ADAPTIVE_THRESH_GAUSSIAN_C,Imgproc.THRESH_BINARY,15, 2);
            //04 NO PROC
            //edgesDetected = original.clone();
            //05 OTSU THRESHOLD
            //Imgproc.threshold(original, edgesDetected,0,255,Imgproc.THRESH_BINARY_INV+Imgproc.THRESH_OTSU);



            //Convert to RGB
            Imgproc.cvtColor(edgesDetected, edgesDetectedRGB, Imgproc.COLOR_GRAY2BGR);
            justEdges = edgesDetectedRGB.clone();//TESTING

            // Standard Hough Line Transform
            Mat lines = new Mat(); // will hold the results of the detection
            //(edgeDetectedImage, outputOfDetection(r,θ), resolution of rho, resolution of theta, threshold (minimum num of intersections)
            Imgproc.HoughLines(edgesDetected, lines, 1, Math.PI / 180.0, 350); // runs the actual detection
            //Imgproc.HoughLines(edgesDetected, lines, 5.0, 4.0, 7); // runs the actual detection

            //make some test images. black and white. row of white row of black. call hough transform. check if its displaying as expected
            //make sure res of 0 is appropriate, try 360
            //expect theta values of 0+-
            //try greyscale image


            for (int i = 0; i < lines.cols(); i++) {
                double rho = lines.get(0, i)[0];
                double theta = lines.get(0, i)[1];
                double cosTheta = Math.cos(theta);
                double sinTheta = Math.sin(theta);
                double x0 = cosTheta * rho;
                double y0 = sinTheta * rho;
                double xpt1 = x0 + 10000 * (-sinTheta);
                double ypt1 = y0 + 10000 * (cosTheta);
                double xpt2 = x0 - 10000 * (-sinTheta);
                double ypt2 = y0 - 10000 * (cosTheta);
                double angle = Core.fastAtan2((float)ypt2 - (float)ypt1, (float)xpt2 - (float)xpt1) * (Math.PI/180.0);
                Point pt1 = new Point(xpt1, ypt1);
                Point pt2 = new Point(xpt2, ypt2);
                Imgproc.line(edgesDetectedRGB, pt1, pt2, new Scalar(0, 0, 255), 3, Imgproc.LINE_AA,0);
                System.out.println("rho: " + rho + '\n' + "theta: " + theta + '\n' + "angle: " + angle);
            }

            /*
            for (int x = 0; x < lines.rows(); x++) {
                double rho = lines.get(x, 0)[0];
                double theta = lines.get(x, 0)[1];
                double cosTheta = Math.cos(theta);
                double sinTheta = Math.sin(theta);                //CONVERT to Cartisean coord
                double x0 = cosTheta * rho;
                double y0 = sinTheta * rho;

                double xpt1 = Math.round(x0 + 10000 * (-sinTheta));
                double ypt1 = Math.round(y0 + 10000 * (cosTheta));
                double xpt2 = Math.round(x0 - 10000 * (-sinTheta));
                double ypt2 = Math.round(y0 - 10000 * (cosTheta));
                double angle = Core.fastAtan2((float)ypt2 - (float)ypt1, (float)xpt2 - (float)xpt1) * (Math.PI/180.0);

                Point pt1 = new Point(xpt1, ypt1);
                Point pt2 = new Point(xpt2, ypt2);
                Imgproc.line(edgesDetectedRGB, pt1, pt2, new Scalar(0, 0, 255), 3, Imgproc.LINE_AA, 0);

                System.out.println("rho: " + rho + '\n' + "theta: " + theta + '\n' + "angle: " + angle);
            }
            */


            //Display Results
            //HighGui.imshow("Source", original);
            HighGui.imshow("Just Edges", justEdges); //TESTING
            HighGui.imshow("Detected Lines (in red) - Standard Hough Line Transform", edgesDetectedRGB);
            //HighGui.imshow("Detected Lines (in red) - Probabilistic Line Transform", edgesDetectedRGBProb);

            // Wait and Exit
            HighGui.waitKey();
            System.exit(0);
        }
        catch(Exception e){
            System.err.println(e);
        }
    }
}
